#!/usr/bin/python -Ou
"""
/**********************************************************************
FILE     : $RCSfile: iRedirector.py,v $
PURPOSE  : squid redirector for icoya
NOTES    : uses redirector_class for custom rules
AUTHOR   : Simon Eisenmann
COPYRIGHT: (c) 2003,2004 by struktur AG
DATE     : 28JAN2003
REVISION : $Revision: 1.7 $
VERSION  : $Id: iRedirector.py,v 1.7 2004/08/05 17:04:50 longsleep Exp $ (Author: $Author: longsleep $)

struktur AG            Phone: +49 711 8966560
Junghansstr. 5         Fax:   +49 711 89665610
70469 Stuttgart        email: info@struktur.de
GERMANY

http://www.struktur.de
http://www.strukturag.com

**********************************************************************/
 iRedirector.py -- a script for squid redirection.
 (a long-running process that uses unbuffered io; hence the -u flag in python)

 NOTE: use redirector_class to define the rules
       redirector_class can be automatically reloaded so you dont have to
       restart squid when the rules were changed.

Optimized by Florian Schulze <florian dot schulze at gmx dot net>

"""
import sys, traceback
from thread import start_new_thread
import redirector_class


# set to 1 to enable logging
debug = 0

# set to 1 to enable squid3 multithreaded redirector support
# NOTE: having this turned on safes a lot of resources
#       only turn this on when having redirect_concurrency in squid.conf 
#       set to something other than 0 
threaded = 0

# the logfile for the redirector log (only used when debug is 1)
logfile = "/var/log/squid/redirector.log"


class SquidRedirector:
    """ iRedirector main base class. """
    
    def __init__(self):
        pass

    def rewrite(self,line):
        if threaded:
            # start new thread
            start_new_thread(rewrite_threaded, (line,), {})
        else:
            # run single threaded
            rewrite(line)
        
    def run(self):
        # wait for stdin input

        line = sys.stdin.readline()[:-1]
        while line:

            if redirector_class.reload_after < 0:
                pass # reload is disabled
            elif redirector_class.reload_after > 0:
                redirector_class.reload_after = redirector_class.reload_after - 1
            else:
                reload(redirector_class)

            # launch rewriting function
            try:
                self.rewrite(line)
            except:
                exc=sys.exc_info()
                log(str(traceback.format_exception(exc[0],exc[1],exc[2])))
                raise 
                # NOTE: this aborts this redirector process
                #       when no redirector is left squid exits as well

            # read new line
            line = sys.stdin.readline()[:-1]


if not debug:
    def log(s):
        pass
else:
    def log(s):
        """ Loggin facility. """
        f = open(logfile,"a")
        f.write(s+'\n')
        f.close()

def rewrite(line):
    """ Splits up the line from squid and gives it to the redirector class. 
        This method can be called in a new thread so one redirector supports
        multiple redirections at the same time. This is a squid3 feature.
    """

    log("request : " + line)

    # format url, src, ident, method
    url,src_address,ident,method=line.split(" ")

    # send through redirector class
    new_url = redirector_class.rewrite(url, src_address)

    if not new_url: 
        # return empty line when redirector didn't return anything
        response=""
    else:
        response = " ".join((new_url,src_address,ident,method))

    # write it back to squid
    sys.stdout.write(response+'\n')
    sys.stdout.flush()

    log("response: " + response)


def rewrite_threaded(line):
    """ Splits up the line from squid and gives it to the redirector class. 
        This method can be called in a new thread so one redirector supports
        multiple redirections at the same time. This is a squid3 feature.
    """

    log("request : " + line)

    # format index, url, src, ident, method
    index,url,src_address,ident,method=line.split(" ")

    # send through redirector class
    new_url = redirector_class.rewrite(url, src_address)

    if not new_url: 
        # return empty line when redirector didn't return anything
        response=index
    else: 
        # give back the index
        response = " ".join((index,new_url,src_address,ident,method))

    # write it back to squid
    sys.stdout.write(response+'\n')
    sys.stdout.flush()

    log("response: " + response)


if __name__ == "__main__":
    sr = SquidRedirector()
    log('Starting SquidRedirector')
    sr.run()
    log('Stopped SquidRedirector')

